﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//Copyright (c) 2009 foldier inc.
//
//Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
//associated documentation files (the "Software"), to deal in the Software without restriction, including 
//without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
//copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the 
//following conditions:
//
//The above copyright notice and this permission notice shall be included in all copies or substantial portions
//of the Software.
//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
//WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
//PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
//ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
//LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
//OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
//IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Data;
using System.Linq;
using System.Xml;
using System.Xml.Linq;
using System.Net;
using System.Configuration;
using System.Collections.Generic;
using System.Text.RegularExpressions;
#if SILVERLIGHT
#else
using System.Web;
using System.Xml.XPath;
#endif

namespace foldier.API
{
	/// <summary>
    /// BaseApi derived class.
    /// Implements link items API calls
	/// </summary>
    public class Items : BaseApi
	{
		#region Protected local members

		protected long m_itemID = -1;

		#endregion

		#region Constructors

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="connector">FoldierOAuthConnector object for authentication</param>
		public Items(FoldierOAuthConnector connector) : base(connector)
		{
		}

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="connector">FoldierOAuthConnector object for authentication</param>
        /// <param name="itemID">Item ID</param>
		public Items(FoldierOAuthConnector connector, long itemID)	: base(connector)
		{
			m_itemID = itemID;
		}

		#endregion

		#region API CALLS

		/// <summary>
        ///   Returns the single item object.
		/// </summary>
        /// <returns>BaseItem object</returns>
		public BaseItem get() { return get(m_itemID); }
		
        /// <summary>
        ///   Returns the single item object within the specified ID
        /// </summary>
        /// <param name="itemID">Item ID</param>
        /// <returns>BaseItem object</returns>
        public BaseItem get(long itemID)
		{
			string result = string.Empty;
			if (itemID <= 0)
				return null;

			result = m_connection.genericApiCall(string.Format("{0:x}/items/{1:x}?out=full&format=xml", m_connection.UserID, itemID), string.Empty);

			return m_connection.resultItem(result);

		}

        /// <summary>
        ///		Returns all the items within the connected account.
        ///		Data returned in Alphabetical order.
        /// </summary>
        /// <returns>ItemLink array</returns>
        public ItemLink[] getAll() { return getAll(0, 20); }
        
        /// <summary>
        ///		Returns all the items within the connected account.
        ///		Data returned in Alphabetical order.
        /// </summary>
        /// <param name="start">Index of first element to return</param>
        /// <param name="count">Number of elements to return</param>
        /// <returns>ItemLink array</returns>
        public ItemLink[] getAll(int start, int count)
        {
            string result = m_connection.genericApiCall(string.Format("{0:x}/items?start={1}&count={2}&format=xml&sort=AlphabeticalAToZ&out=full", m_connection.UserID, start, count)
                                                       , string.Empty);

            return m_connection.resultItemLinks(result);
        }

        /// <summary>
        ///     Search for ItemLinks matching query expression
        /// </summary>
        /// <param name="query">Search query</param>
        /// <returns>ItemLink array</returns>
        public ItemLink[] searchLinks(string query) { return searchLinks(m_itemID, query, 0, 20); }
        
        /// <summary>
        ///     Search for ItemLinks matching query expression within a specific ID
        /// </summary>
        /// <param name="itemID">Item ID</param>
        /// <param name="query">Search query</param>
        /// <returns>ItemLink array</returns>
        public ItemLink[] searchLinks(long itemID, string query) { return searchLinks(m_itemID, query, 0, 20); }
        
        /// <summary>
        ///     Search for ItemLinks matching query expression
        /// </summary>
        /// <param name="query">Search query</param>
        /// <param name="start">Index of first element to return</param>
        /// <param name="count">Number of elements to return</param>
        /// <returns>ItemLink array</returns>
        public ItemLink[] searchLinks(string query, int start, int count) { return searchLinks(m_itemID, query, start, count); }
        
        /// <summary>
        ///     Search for ItemLinks matching query expression
        /// </summary>
        /// <param name="itemID">Item ID</param>
        /// <param name="query">Search query</param>
        /// <param name="start">Index of first element to return</param>
        /// <param name="count">Number of elements to return</param>
        /// <returns>ItemLink array</returns>
        public ItemLink[] searchLinks(long itemID, string query, int start, int count)
        {
            string result = string.Empty;
            if (itemID > 0)
                result = m_connection.genericApiCall(string.Format("{0:x}/items/{1:x}?q={2}&start={3}&count={4}&format=xml&out=fullcontent&sort=ModifiedNewToOld"
                                                    , m_connection.UserID, itemID, FoldierOauth.UriEncode(query), start, count), string.Empty);
            else
                result = m_connection.genericApiCall(string.Format("{0:x}/items?q={1}&start={2}&count={3}&format=xml&out=fullcontent&sort=ModifiedNewToOld"
                                                    , m_connection.UserID, FoldierOauth.UriEncode(query), start, count), string.Empty);

            return m_connection.resultItemLinks(result);
        }

		#endregion

	}
}
